<?php
/*
Plugin Name: QR Code Generator
Description: Generates QR codes for posts, pages, and products.
Version: 1.1
Author: EasyToolsWP.com
*/

// Prevent direct access to the file
if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

// Add a subpage under 'Tools'
add_action('admin_menu', 'qr_code_generator_menu');
function qr_code_generator_menu() {
    add_submenu_page(
        'tools.php', 
        'QR Code Generator Settings', 
        'QR Code Generator', 
        'manage_options', 
        'qr-code-generator-settings', 
        'qr_code_generator_settings_page'
    );
}

// Settings page content
function qr_code_generator_settings_page() {
    ?>
    <div class="wrap">
        <h1>QR Code Generator Settings</h1>
        <form method="post" action="options.php">
            <?php
            settings_fields('qr_code_generator_settings_group');
            do_settings_sections('qr-code-generator-settings');
            submit_button();
            ?>
        </form>
    </div>
    <?php
}

// Register and add settings
add_action('admin_init', 'qr_code_generator_settings_init');
function qr_code_generator_settings_init() {
    register_setting('qr_code_generator_settings_group', 'qr_code_display_pages');
    register_setting('qr_code_generator_settings_group', 'qr_code_display_posts');
    register_setting('qr_code_generator_settings_group', 'qr_code_display_products');
    register_setting('qr_code_generator_settings_group', 'qr_code_size');
    register_setting('qr_code_generator_settings_group', 'qr_code_position');

    add_settings_section(
        'qr_code_generator_settings_section', 
        'Display Settings', 
        null, 
        'qr-code-generator-settings'
    );

    add_settings_field(
        'qr_code_display_pages', 
        'Display on Pages', 
        'qr_code_display_pages_render', 
        'qr-code-generator-settings', 
        'qr_code_generator_settings_section'
    );

    add_settings_field(
        'qr_code_display_posts', 
        'Display on Posts', 
        'qr_code_display_posts_render', 
        'qr-code-generator-settings', 
        'qr_code_generator_settings_section'
    );

    add_settings_field(
        'qr_code_display_products', 
        'Display on Products', 
        'qr_code_display_products_render', 
        'qr-code-generator-settings', 
        'qr_code_generator_settings_section'
    );

    add_settings_field(
        'qr_code_size', 
        'QR Code Size (px)', 
        'qr_code_size_render', 
        'qr-code-generator-settings', 
        'qr_code_generator_settings_section'
    );

    add_settings_field(
        'qr_code_position', 
        'QR Code Position', 
        'qr_code_position_render', 
        'qr-code-generator-settings', 
        'qr_code_generator_settings_section'
    );
}

function qr_code_display_pages_render() {
    $value = get_option('qr_code_display_pages', '');
    ?>
    <input type="checkbox" name="qr_code_display_pages" value="1" <?php checked(1, $value, true); ?> />
    <?php
}

function qr_code_display_posts_render() {
    $value = get_option('qr_code_display_posts', '');
    ?>
    <input type="checkbox" name="qr_code_display_posts" value="1" <?php checked(1, $value, true); ?> />
    <?php
}

function qr_code_display_products_render() {
    $value = get_option('qr_code_display_products', '');
    ?>
    <input type="checkbox" name="qr_code_display_products" value="1" <?php checked(1, $value, true); ?> />
    <?php
}

function qr_code_size_render() {
    $value = get_option('qr_code_size', '150');
    ?>
    <input type="number" name="qr_code_size" value="<?php echo esc_attr($value); ?>" />
    <?php
}

function qr_code_position_render() {
    $value = get_option('qr_code_position', 'left');
    ?>
    <select name="qr_code_position">
        <option value="left" <?php selected($value, 'left'); ?>>Float left</option>
        <option value="right" <?php selected($value, 'right'); ?>>Float right</option>
        <option value="description" <?php selected($value, 'description'); ?>>Description</option>
    </select>
    <?php
}

// Conditionally add QR code to footer
add_action('wp_footer', 'conditionally_add_qr_code_to_footer');
function conditionally_add_qr_code_to_footer() {
    if (!is_singular()) {
        return;
    }

    global $post;

    // Check if QR code should be hidden
    $hide_qr_code = get_post_meta($post->ID, '_hide_qr_code', true);

    if ($hide_qr_code) {
        return;
    }

    // Check if QR code should be displayed based on plugin settings
    $should_display = false;
    if (is_page() && get_option('qr_code_display_pages')) {
        $should_display = true;
    } elseif (is_single() && get_option('qr_code_display_posts')) {
        $should_display = true;
    } elseif (is_product() && get_option('qr_code_display_products')) {
        $should_display = true;
    }

    // If the display setting is disabled, do not display QR code
    if (!$should_display) {
        return;
    }

    $url = get_permalink();
    $size = get_option('qr_code_size', '150');
    $position = get_option('qr_code_position', 'left');
    $qr_code_url = 'https://api.qrserver.com/v1/create-qr-code/?size=' . $size . 'x' . $size . '&data=' . urlencode($url);

    if ($position === 'description') {
        return; // Add QR code in a different hook for product descriptions
    }

    $qr_code_img = '<div class="qr-code" style="position:fixed; bottom:10px; ' . esc_attr($position) . ':10px; z-index:1000;">
                        <img src="' . esc_url($qr_code_url) . '" alt="QR Code" style="width: ' . esc_attr($size) . 'px; height: ' . esc_attr($size) . 'px;" />
                    </div>';
    
    echo $qr_code_img;
}

// Add QR code under product meta
add_action('woocommerce_single_product_summary', 'conditionally_add_qr_code_to_product_meta', 25);
function conditionally_add_qr_code_to_product_meta() {
    global $post;

    // Check if QR code should be hidden
    $hide_qr_code = get_post_meta($post->ID, '_hide_qr_code', true);
    if ($hide_qr_code) {
        return;
    }

    // Display QR code only if plugin setting is enabled and position is description
    if (get_option('qr_code_display_products') && get_option('qr_code_position') === 'description') {
        $url = get_permalink();
        $size = get_option('qr_code_size', '150');
        $qr_code_url = 'https://api.qrserver.com/v1/create-qr-code/?size=' . $size . 'x' . $size . '&data=' . urlencode($url);

        $qr_code_img = '<div class="qr-code" style="margin-top: 20px;">
                            <img src="' . esc_url($qr_code_url) . '" alt="QR Code" style="width: ' . esc_attr($size) . 'px; height: ' . esc_attr($size) . 'px;" />
                        </div>';

        echo $qr_code_img;
    }
}

// Add CSS to ensure proper QR code styling
add_action('wp_head', 'qr_code_generator_styles');
function qr_code_generator_styles() {
    echo '<style>
        .qr-code {
            background-color: #fff;
            padding: 8px;
            border-radius: 10px;
        }
    </style>';
}

// Add metabox to individual posts, pages, and products
add_action('add_meta_boxes', 'qr_code_generator_add_meta_box');
function qr_code_generator_add_meta_box() {
    global $post;
    $post_type = get_post_type($post);

    // Check if QR code should be displayed for the current post type
    if (($post_type === 'page' && get_option('qr_code_display_pages')) ||
        ($post_type === 'post' && get_option('qr_code_display_posts')) ||
        ($post_type === 'product' && get_option('qr_code_display_products'))) {
        
        add_meta_box(
            'qr_code_generator_meta_box',
            'QR Code Settings',
            'qr_code_generator_meta_box_callback',
            array('post', 'page', 'product'),
            'side'
        );
    }
}

function qr_code_generator_meta_box_callback($post) {
    wp_nonce_field('qr_code_generator_save_meta_box_data', 'qr_code_generator_meta_box_nonce');

    $value = get_post_meta($post->ID, '_hide_qr_code', true);

    ?>
    <label for="qr_code_hide">
        <input type="checkbox" name="qr_code_hide" id="qr_code_hide" value="1" <?php checked(1, $value, true); ?> />
        Hide QR Code
    </label>
    <?php
}

add_action('save_post', 'qr_code_generator_save_meta_box_data');
function qr_code_generator_save_meta_box_data($post_id) {
    if (!isset($_POST['qr_code_generator_meta_box_nonce'])) {
        return;
    }

    if (!wp_verify_nonce($_POST['qr_code_generator_meta_box_nonce'], 'qr_code_generator_save_meta_box_data')) {
        return;
    }

    if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) {
        return;
    }

    if (!current_user_can('edit_post', $post_id)) {
        return;
    }

    if (isset($_POST['qr_code_hide'])) {
        update_post_meta($post_id, '_hide_qr_code', 1);
    } else {
        delete_post_meta($post_id, '_hide_qr_code');
    }
}

// Shortcode to display QR code
function etwp_qr_code_shortcode($atts) {
    global $post;

    if (!is_singular() || is_archive()) {
        return '';
    }

    // Check if QR code should be hidden
    $hide_qr_code = get_post_meta($post->ID, '_hide_qr_code', true);
    if ($hide_qr_code) {
        return '';
    }

    // Check if QR code should be displayed based on plugin settings
    $should_display = false;
    if (is_page() && get_option('qr_code_display_pages')) {
        $should_display = true;
    } elseif (is_single() && get_option('qr_code_display_posts')) {
        $should_display = true;
    } elseif (is_product() && get_option('qr_code_display_products')) {
        $should_display = true;
    }

    // If the display setting is disabled, do not display QR code
    if (!$should_display) {
        return '';
    }

    $url = get_permalink();
    $size = get_option('qr_code_size', '150');
    $qr_code_url = 'https://api.qrserver.com/v1/create-qr-code/?size=' . $size . 'x' . $size . '&data=' . urlencode($url);

    return '<div class="qr-code">
                <img src="' . esc_url($qr_code_url) . '" alt="QR Code" style="width: ' . esc_attr($size) . 'px; height: ' . esc_attr($size) . 'px;" />
            </div>';
}
add_shortcode('etwp_qr_code', 'etwp_qr_code_shortcode');
?>
